<?php
/**
 * Omnisend AJAX Functions
 *
 * @package OmnisendPlugin
 */

defined( 'ABSPATH' ) || exit;

add_action( 'wp_ajax_track_connection_event', 'track_connection_event' );
add_action( 'wp_ajax_nopriv_track_connection_event', 'track_connection_event' );

function track_connection_event() {
	// phpcs:disable WordPress.Security.NonceVerification
	if ( isset( $_POST['event_name'] ) ) {
		Omnisend_Plugin_Event::connection_event( sanitize_text_field( wp_unslash( $_POST['event_name'] ) ) );
	}
	exit;
	// phpcs:enable
}

add_action( 'wp_ajax_omnisend_identify', 'hook_omnisend_ajax_save_email' );
add_action( 'wp_ajax_nopriv_omnisend_identify', 'hook_omnisend_ajax_save_email' );

function hook_omnisend_ajax_save_email() {
	// phpcs:disable WordPress.Security.NonceVerification
	Omnisend_Logger::hook();

	$email      = ! empty( $_GET['email'] ) ? sanitize_email( wp_unslash( $_GET['email'] ) ) : '';
	$contact_id = ! empty( $_GET['contact_id'] ) ? sanitize_text_field( wp_unslash( $_GET['contact_id'] ) ) : '';

	Omnisend_Logger::debug( "hook_omnisend_ajax_save_email - received email: $email, contact_id: $contact_id" );
	Omnisend_Ajax::identify_by_email( $email, $contact_id );

	exit;
	// phpcs:enable
}

add_action( 'wp_ajax_track_started_checkout_event', 'track_started_checkout_event' );
add_action( 'wp_ajax_nopriv_track_started_checkout_event', 'track_started_checkout_event' );

function track_started_checkout_event() {
	// phpcs:disable WordPress.Security.NonceVerification
	Omnisend_Logger::hook();
	$email = ! empty( $_GET['email'] ) ? sanitize_email( wp_unslash( $_GET['email'] ) ) : '';
	Omnisend_Logger::debug( "track_started_checkout_event - received email: $email" );
	Omnisend_Cart_Event::started_checkout( $email );
	exit;
	// phpcs:enable
}

class Omnisend_Ajax {
	/**
	 * @return Omnisend_Operation_Status
	 */
	public static function identify_by_email( $email, $contact_id ) {
		if ( ! Omnisend_Manager::is_setup() ) {
			return Omnisend_Operation_Status::error( 'Omnisend is not setup' );
		}

		if ( ! filter_var( $email, FILTER_VALIDATE_EMAIL ) ) {
			return Omnisend_Operation_Status::error( 'Incorrect request (email)' );
		}

		if ( Omnisend_Contact_Resolver::update_by_email( $email ) ) {
			self::trigger_cart_push();

			return Omnisend_Operation_Status::success();
		}

		$api_url     = OMNISEND_API_URL . 'contacts';
		$curl_result = Omnisend_Helper::omnisend_api( $api_url, 'POST', self::generate_contact_payload( $email, $contact_id ) );
		if ( $curl_result['code'] < 200 || $curl_result['code'] >= 300 ) {
			Omnisend_Logger::log( 'warn', 'contacts', $api_url, 'Unable to push contact ' . $email . ' to Omnisend.' . $curl_result['response'] );

			return Omnisend_Operation_Status::error( 'Unable to create contact (api error)' );
		}
		Omnisend_Logger::log( 'info', 'contacts', $api_url, 'Contact ' . $email . ' was successfully pushed to Omnisend.' );

		$response = json_decode( $curl_result['response'], true );
		if ( empty( $response['contactID'] ) ) {
			Omnisend_Logger::log( 'warn', 'contacts', $api_url, 'Unable to push contact ' . $email . ' to Omnisend. Unexpected API response: ' . $curl_result['response'] );

			return Omnisend_Operation_Status::error( 'Unable to identify contact (api error)' );
		}

		Omnisend_Contact_Resolver::update_by_email_and_contact_id( $email, $response['contactID'] );
		self::trigger_cart_push();

		return Omnisend_Operation_Status::success();
	}

	private static function generate_contact_payload( $email, $contact_id ) {
		$tag = get_option( 'omnisend_contact_tag', null );

		return array(
			'contactID'  => $contact_id,
			'email'      => $email,
			'status'     => 'nonSubscribed',
			'statusDate' => date( DATE_ATOM ),
			'tags'       => $tag ? array( 'source: woocommerce', $tag ) : array( 'source: woocommerce' ),
		);
	}

	private static function trigger_cart_push() {
		if ( ! WC()->cart->is_empty() ) {
			Omnisend_Manager::push_cart_to_omnisend();
		}
	}
}

class Omnisend_Operation_Status {
	/**
	 * @var bool
	 */
	private $success;

	/**
	 * @var string
	 */
	private $message;

	/**
	 * @return Omnisend_Operation_Status
	 */
	public static function success() {
		return new Omnisend_Operation_Status( true, '' );
	}

	/**
	 * @param $message
	 *
	 * @return Omnisend_Operation_Status
	 */
	public static function error( $message ) {
		return new Omnisend_Operation_Status( false, $message );
	}

	/**
	 * @param $success bool
	 * @param $message string
	 */
	private function __construct( $success, $message ) {
		$this->success = $success;
		$this->message = $message;
	}

	/**
	 * @return string
	 */
	public function to_string() {
		return json_encode(
			array(
				'success' => $this->success,
				'message' => $this->message,
			)
		);
	}
}
