<?php
/**
 * Omnisend Sync Stats Repository
 *
 * @package OmnisendPlugin
 */

defined( 'ABSPATH' ) || exit;

const POST_STATUS_PUBLISH = 'publish';

const POST_TYPE_SHOP_ORDER = 'shop_order';
const POST_TYPE_PRODUCT    = 'product';

class Omnisend_Sync_Stats_Repository {

	public static function count_item( $endpoint, $inc = 1 ) {
		$key   = 'omnisend_' . $endpoint . '_sync_count';
		$count = intval( get_option( $key ) ) + $inc;
		update_option( $key, $count );
	}

	/**
	 * @return Omnisend_All_Sync_Stats
	 */
	public function get_all_stats() {
		return new Omnisend_All_Sync_Stats(
			$this->get_contact_stats(),
			$this->get_order_stats(),
			$this->get_product_stats(),
			$this->get_category_stats(),
			intval( get_option( 'omnisend_carts_sync_count' ) )
		);
	}

	/**
	 * @return Omnisend_Sync_Stats
	 */
	private function get_contact_stats() {
		$synced = array(
			'meta_query' => array( // phpcs:ignore WordPress.DB.SlowDBQuery.slow_db_query_meta_query
				'relation' => 'OR',
				array(
					'key'     => Omnisend_Sync::FIELD_NAME,
					'compare' => 'LIKE',
					'value'   => '20',
				),
			),
		);

		$not_synced = array(
			'meta_query' => array( // phpcs:ignore WordPress.DB.SlowDBQuery.slow_db_query_meta_query
				'relation' => 'OR',
				array(
					'key'     => Omnisend_Sync::FIELD_NAME,
					'compare' => 'NOT EXISTS',
					'value'   => '',
				),
			),
		);

		$skipped = array(
			'meta_query' => array( // phpcs:ignore WordPress.DB.SlowDBQuery.slow_db_query_meta_query
				'relation' => 'OR',
				array(
					'key'   => Omnisend_Sync::FIELD_NAME,
					'value' => Omnisend_Sync::STATUS_SKIPPED,
				),
			),
		);

		$error = array(
			'meta_query' => array( // phpcs:ignore WordPress.DB.SlowDBQuery.slow_db_query_meta_query
				'relation' => 'OR',
				array(
					'key'   => Omnisend_Sync::FIELD_NAME,
					'value' => Omnisend_Sync::STATUS_ERROR,
				),
			),
		);

		global $wpdb;

		$unique_count = $wpdb->get_var( "SELECT COUNT(DISTINCT user_email) FROM $wpdb->users WHERE user_email != '' AND user_email IS NOT null" );

		return new Omnisend_Sync_Stats(
			$this->get_user_count( array() ),
			$unique_count,
			$this->get_user_count( $synced ),
			$this->get_user_count( $not_synced ),
			$this->get_user_count( $skipped ),
			$this->get_user_count( $error )
		);
	}

	/**
	 * @param array $query_params
	 *
	 * @return int
	 */
	private function get_user_count( $query_params ) {
		$general = array(
			'count_total' => true,
			'number'      => 1,
		);

		return ( new WP_User_Query( array_merge( $general, $query_params ) ) )->get_total();
	}



	/**
	 * @return Omnisend_Sync_Stats
	 */
	private function get_order_stats() {
		return $this->get_stats_from_posts( POST_TYPE_SHOP_ORDER );
	}

	/**
	 * @return Omnisend_Sync_Stats
	 */
	private function get_stats_from_posts( $post_type, $post_status = '' ) {
		global $wpdb;

		$sync_field          = Omnisend_Sync::FIELD_NAME;
		$sync_status_skipped = Omnisend_Sync::STATUS_SKIPPED;
		$sync_status_error   = Omnisend_Sync::STATUS_ERROR;

		$status_sql = $post_status ? $wpdb->prepare( ' AND post_status = %s', $post_status ) : '';

		$sql = $wpdb->prepare(
			"SELECT 
				COUNT(ID) as total,
				SUM(IF(meta.meta_value LIKE %s, 1, 0)) as synced,
				SUM(IF(meta.meta_value IS NULL, 1, 0)) as notSynced,
				SUM(IF(meta.meta_value = %s, 1, 0)) as skipped,
				SUM(IF(meta.meta_value = %s, 1, 0)) as error
			FROM $wpdb->posts AS post
			LEFT JOIN $wpdb->postmeta AS meta ON meta.post_id = post.ID AND meta.meta_key = %s
			WHERE post_type = %s %s
			GROUP by '1';
			",
			'20%',
			$sync_status_skipped,
			$sync_status_error,
			$sync_field,
			$post_type,
			$status_sql
		);

		// phpcs:ignore WordPress.DB.PreparedSQL.NotPrepared
		$info = $wpdb->get_row( $sql, ARRAY_A );

		return new Omnisend_Sync_Stats(
			$this->get_array_field_or_zero( $info, 'total' ),
			$this->get_array_field_or_zero( $info, 'total' ),
			$this->get_array_field_or_zero( $info, 'synced' ),
			$this->get_array_field_or_zero( $info, 'notSynced' ),
			$this->get_array_field_or_zero( $info, 'skipped' ),
			$this->get_array_field_or_zero( $info, 'error' )
		);
	}

	private function get_array_field_or_zero( $array, $field ) {
		return ! empty( $array[ $field ] ) ? $array[ $field ] : 0;
	}

	/**
	 * @return Omnisend_Sync_Stats
	 */
	private function get_product_stats() {
		return $this->get_stats_from_posts( POST_TYPE_PRODUCT, POST_STATUS_PUBLISH );
	}

	/**
	 * @return Omnisend_Sync_Stats
	 */
	private function get_category_stats() {
		global $wpdb;

		$sync_field          = Omnisend_Sync::FIELD_NAME;
		$sync_status_skipped = Omnisend_Sync::STATUS_SKIPPED;
		$sync_status_error   = Omnisend_Sync::STATUS_ERROR;

		$sql = $wpdb->prepare(
			"SELECT
				COUNT(t.term_id) as total,
				SUM(IF(m2.meta_value LIKE %s, 1, 0)) as synced,
				SUM(IF(m2.meta_value IS NULL, 1, 0)) as notSynced,
				SUM(IF(m2.meta_value = %s, 1, 0)) as skipped,
				SUM(IF(m2.meta_value = %s, 1, 0)) as error
			FROM $wpdb->terms AS t
			LEFT JOIN $wpdb->termmeta AS m1 ON ( t.term_id = m1.term_id )
			LEFT JOIN $wpdb->termmeta AS m2 ON ( t.term_id = m2.term_id AND m2.meta_key = %s )
			INNER JOIN $wpdb->term_taxonomy AS tt ON t.term_id = tt.term_id WHERE tt.taxonomy IN ('product_cat') AND ( m1.meta_key = 'order' OR m1.meta_key IS NULL )
			GROUP by '1';
			",
			'20%',
			$sync_status_skipped,
			$sync_status_error,
			$sync_field
		);

		// phpcs:ignore WordPress.DB.PreparedSQL.NotPrepared
		$info = $wpdb->get_row( $sql, ARRAY_A );

		return new Omnisend_Sync_Stats(
			$this->get_array_field_or_zero( $info, 'total' ),
			$this->get_array_field_or_zero( $info, 'total' ),
			$this->get_array_field_or_zero( $info, 'synced' ),
			$this->get_array_field_or_zero( $info, 'notSynced' ),
			$this->get_array_field_or_zero( $info, 'skipped' ),
			$this->get_array_field_or_zero( $info, 'error' )
		);
	}
}

class Omnisend_Sync_Stats {
	public $total;
	public $unique;
	public $synced;
	public $not_synced;
	public $skipped;
	public $error;

	public function __construct( $total, $unique, $synced, $not_synced, $skipped, $error ) {
		$this->total      = $total;
		$this->unique     = $unique;
		$this->synced     = $synced;
		$this->not_synced = $not_synced;
		$this->skipped    = $skipped;
		$this->error      = $error;
	}
}

class Omnisend_All_Sync_Stats {
	/**
	 * @var Omnisend_Sync_Stats
	 */
	public $contacts;
	/**
	 * @var Omnisend_Sync_Stats
	 */
	public $orders;
	/**
	 * @var Omnisend_Sync_Stats
	 */
	public $products;
	/**
	 * @var Omnisend_Sync_Stats
	 */
	public $categories;
	/**
	 * @var int
	 */
	public $carts;

	public function __construct( Omnisend_Sync_Stats $contacts, Omnisend_Sync_Stats $orders, Omnisend_Sync_Stats $products, Omnisend_Sync_Stats $categories, $carts ) {
		$this->contacts   = $contacts;
		$this->orders     = $orders;
		$this->products   = $products;
		$this->categories = $categories;
		$this->carts      = $carts;
	}
}
