<?php
/**
 * Omnisend Contact Class
 *
 * @package OmnisendPlugin
 */

defined( 'ABSPATH' ) || exit;

class Omnisend_Contact {

	public $email;
	public $phone;
	public $status;
	public $country;
	public $state;
	public $city;
	public $address;
	// phpcs:disable WordPress.NamingConventions.ValidVariableName.PropertyNotSnakeCase
	public $firstName;
	public $lastName;
	public $countryCode;
	public $postalCode;
	public $statusDate;
	// phpcs:enable
	public $state_code;
	public $tags;
	public $lists;

	public static function create( $user ) {
		try {
			return new Omnisend_Contact( $user );
		} catch ( Omnisend_Empty_Required_Fields_Exception $exception ) {
			return null;
		}
	}

	/**
	 * @throws Omnisend_Empty_Required_Fields_Exception
	 */
	private function __construct( $user ) {
		if ( empty( $user ) ) {
			throw new Omnisend_Empty_Required_Fields_Exception();
		}

		$email = $user->user_email;
		if ( filter_var( $email, FILTER_VALIDATE_EMAIL ) ) {
			$this->email = $email;
		}

		// phpcs:disable WordPress.Security.NonceVerification
		// phpcs:disable WordPress.NamingConventions.ValidVariableName.UsedPropertyNotSnakeCase
		if ( get_user_meta( $user->ID, 'first_name', true ) !== '' ) {
			$this->firstName = get_user_meta( $user->ID, 'first_name', true );
		} elseif ( $_POST && isset( $_POST['billing_first_name'] ) ) {
			$this->firstName = sanitize_text_field( wp_unslash( $_POST['billing_first_name'] ) );
		} elseif ( get_user_meta( $user->ID, 'shipping_first_name', true ) !== '' ) {
			$this->firstName = get_user_meta( $user->ID, 'shipping_first_name', true );
		} elseif ( get_user_meta( $user->ID, 'billing_first_name', true ) !== '' ) {
			$this->firstName = get_user_meta( $user->ID, 'billing_first_name', true );
		} else {
			$this->firstName = $user->display_name;
		}

		if ( get_user_meta( $user->ID, 'last_name', true ) != '' ) {
			$this->lastName = get_user_meta( $user->ID, 'last_name', true );
		} elseif ( $_POST && isset( $_POST['billing_last_name'] ) ) {
			$this->lastName = sanitize_text_field( wp_unslash( $_POST['billing_last_name'] ) );
		} elseif ( get_user_meta( $user->ID, 'shipping_last_name', true ) != '' ) {
			$this->lastName = get_user_meta( $user->ID, 'shipping_last_name', true );
		} elseif ( get_user_meta( $user->ID, 'billing_last_name', true ) != '' ) {
			$this->lastName = get_user_meta( $user->ID, 'billing_last_name', true );
		}
		if ( $_POST && isset( $_POST['shipping_country'] ) && Omnisend_Helper::valid_country_code( sanitize_text_field( wp_unslash( $_POST['shipping_country'] ) ) ) ) {
			$this->countryCode = sanitize_text_field( wp_unslash( $_POST['shipping_country'] ) );
		} elseif ( $_POST && isset( $_POST['billing_country'] ) && Omnisend_Helper::valid_country_code( sanitize_text_field( wp_unslash( $_POST['billing_country'] ) ) ) ) {
			$this->countryCode = sanitize_text_field( wp_unslash( $_POST['billing_country'] ) );
		} elseif ( ( $billing_country = get_user_meta( $user->ID, 'billing_country', true ) ) !== '' && Omnisend_Helper::valid_country_code( $billing_country ) ) {
			$this->countryCode = $billing_country;
		} elseif ( ( $shipping_country = get_user_meta( $user->ID, 'shipping_country', true ) ) !== '' && Omnisend_Helper::valid_country_code( $shipping_country ) ) {
			$this->countryCode = $shipping_country;
		}

		if ( ! empty( $this->countryCode ) ) {
			$this->country = WC()->countries->countries[ $this->countryCode ];
		}

		if ( ( $billing_state = get_user_meta( $user->ID, 'billing_state', true ) ) != '' ) {
			$this->state = $billing_state;
		} elseif ( ( $shipping_state = get_user_meta( $user->ID, 'shipping_state', true ) ) != '' ) {
			$this->state = $shipping_state;
		} elseif ( $_POST && isset( $_POST['billing_state'] ) ) {
			$this->state = sanitize_text_field( wp_unslash( $_POST['billing_state'] ) );
		}

		if ( ! empty( $this->state ) && ! empty( $this->countryCode ) ) {
			$states = WC()->countries->get_states( $this->countryCode );
			if ( ! empty( $states[ $this->state ] ) ) {
				$this->state_code = $this->state;
				$this->state      = $states[ $this->state ];
			}
		}

		if ( ( $billing_city = get_user_meta( $user->ID, 'billing_city', true ) ) != '' ) {
			$this->city = $billing_city;
		} elseif ( ( $shipping_city = get_user_meta( $user->ID, 'shipping_city', true ) ) != '' ) {
			$this->city = $shipping_city;
		} elseif ( $_POST && isset( $_POST['billing_city'] ) ) {
			$this->city = sanitize_text_field( wp_unslash( $_POST['billing_city'] ) );
		}

		$address = '';
		if ( ( $address1 = get_user_meta( $user->ID, 'billing_address_1', true ) ) != '' ) {
			$address .= $address1;
		} elseif ( ( $address1 = get_user_meta( $user->ID, 'shipping_address_1', true ) ) != '' ) {
			$address .= $address1;
		} elseif ( $_POST && isset( $_POST['billing_address_1'] ) ) {
			$address .= sanitize_text_field( wp_unslash( $_POST['billing_address_1'] ) );
		}

		if ( ( $address2 = get_user_meta( $user->ID, 'billing_address_2', true ) ) != '' ) {
			$address .= $address2;
		} elseif ( ( $address2 = get_user_meta( $user->ID, 'shipping_address_2', true ) ) != '' ) {
			$address .= $address2;
		} elseif ( $_POST && isset( $_POST['billing_address_2'] ) ) {
			$address .= sanitize_text_field( wp_unslash( $_POST['billing_address_2'] ) );
		}
		$this->address = $address;

		if ( ( $postal_code = get_user_meta( $user->ID, 'billing_postcode', true ) ) != '' ) {
			$this->postalCode = $postal_code;
		} elseif ( ( $postal_code = get_user_meta( $user->ID, 'shipping_postcode', true ) ) != '' ) {
			$this->postalCode = $postal_code;
		} elseif ( $_POST && isset( $_POST['billing_postcode'] ) ) {
			$this->postalCode = sanitize_text_field( wp_unslash( $_POST['billing_postcode'] ) );
		}

		$phone_number = get_user_meta( $user->ID, 'billing_phone', true );
		if ( $phone_number ) {
			$this->phone = $phone_number;
		}

		// Set tag/list.
		$list_id = get_option( 'omnisend_list_id', null );
		if ( $list_id ) {
			$this->lists[] = array( 'listID' => $list_id );
		}
		$this->tags = array();
		$tag        = get_option( 'omnisend_contact_tag', null );
		if ( $tag != '' ) {
			$this->tags[] = $tag;
		} else {
			$list_id = get_option( 'omnisend_list_id', null );
			if ( $list_id ) {
				$this->lists[] = array( 'listID' => $list_id );
			}
		}
		$this->tags[] = 'source: woocommerce';

		$this->status = 'nonSubscribed';

		if ( $user->user_registered ) {
			$this->statusDate = date( DATE_ATOM, strtotime( $user->user_registered ) );
		} else {
			$this->statusDate = date( DATE_ATOM, time() );
		}
		if ( empty( $this->email ) || empty( $this->status ) || empty( $this->statusDate ) ) {
			throw new Omnisend_Empty_Required_Fields_Exception();
		}
		// phpcs:enable
	}
}
